package com.dsic.gj_erp.handler;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateRange;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.SpringContextHolder;
import com.dsic.gj_erp.bean.jcsj.DmBom;
import com.dsic.gj_erp.bean.jcsj.EmGcrl;
import com.dsic.gj_erp.bean.jcsj.EmSbcnp;
import com.dsic.gj_erp.bean.jcsj.EmSbjbb;
import com.dsic.gj_erp.bean.jhgk.DmSygdMx;
import com.dsic.gj_erp.handler.dto.RealCapacity;
import com.dsic.gj_erp.service.jcsj.EmGcrlService;
import com.dsic.gj_erp.service.jcsj.EmSbcnpService;
import com.dsic.gj_erp.service.jcsj.EmSbjbbService;
import lombok.Getter;
import lombok.Setter;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Getter
@Setter
public class ComputePlan {

    private static ComputePlan instance = null;

    List<Object> workerList;

    Map<String, EmSbcnp> baseCapacity;

    Map<String,Integer> empSizeByDay;

    List<EmGcrl> gcrlList;

    JSONObject realCapacityMap;

    List<EmSbjbb> deviceList;

    JSONObject realDeviceList;

    List<DmSygdMx> sygdMxList;

    Map<String,List<DmBom>> dmBomList;

    /**
     * 计算实际产能,默认未来三个月
     */
    public static void computeRealCapacity() {
        Date from = DateUtil.beginOfMonth(DateUtil.offsetMonth(new Date(), 1));
        Date to = DateUtil.beginOfMonth(DateUtil.offsetMonth(new Date(), 4));
        computeRealCapacity(from, to);
    }

    /**
     * 计算制定日期范围内每天实际产能
     *
     * @param from 开始日期
     * @param to   结束日期
     */
    public static void computeRealCapacity(Date from, Date to) {
        JSONObject _realCapacity = instance().realCapacityMap = new JSONObject();

        //加载工厂日历
        initGcrl();
        //加载设备信息
        initDevice();
        //计算周期内按天分解的实际可用设备产能
        initRealDeviceList(from, to);

        //可用设备
        JSONObject realDeviceList = instance().realDeviceList;

        List<EmGcrl> gcrlList = instance().gcrlList.stream()
                .filter(item -> "1".equals(item.getXxr()))
                .filter(item -> DateUtil.isIn(DateUtil.parse(item.getGl(), "yyyy/MM/dd"), from, to))
                .collect(Collectors.toList());

        gcrlList.forEach(gcrl -> {
            if (realDeviceList == null) {
                return;
            }
            //日可用设备
            for (int i = 0; i < realDeviceList.getJSONArray(gcrl.getGl()).size(); i++) {
                JSONObject deviceJson = realDeviceList.getJSONArray(gcrl.getGl()).getJSONObject(i);

                //日实际产能
                //todo 设备和产能对应关系,找出当前设备产能
                RealCapacity realCapacity = initRealCapacity(gcrl, deviceJson.getString("sbbh"), new EmSbcnp());
                if (_realCapacity.containsKey(gcrl.getGl())) {
                    _realCapacity.getJSONObject(gcrl.getGl()).put(realCapacity.getDeviceName(), realCapacity.getCapacity());
                } else {
                    JSONObject object = new JSONObject();
                    object.put(realCapacity.getDeviceName(), realCapacity.getCapacity());
                    _realCapacity.put(gcrl.getGl(), object);
                }
            }
        });

    }

    /**
     * 加载工厂日历
     */
    public static void initGcrl() {
        if (instance().gcrlList == null){
            instance().gcrlList = getBean(EmGcrlService.class).getList(new JSONObject());
        }
    }

    public static void initDeviceCapacity() {
        if (instance().baseCapacity == null) {
            instance().baseCapacity = getBean(EmSbcnpService.class).getMap();
        }
    }

    public static void initRealDeviceList(Date from, Date to) {
        initRealDeviceList(from,to,false);
    }

    /**
     * 按日期分解可用设备
     * @param from 开始日期
     * @param to 结束日期
     * @param flag 强制重置
     */
    public static void initRealDeviceList(Date from, Date to,boolean flag) {
        if (instance().realDeviceList == null||flag) {
            JSONObject realDeviceList = instance().realDeviceList = new JSONObject();
            List<EmSbjbb> list = instance().deviceList;
            DateRange dateRange = new DateRange(from, to, DateField.DAY_OF_YEAR);
            List<EmSbjbb> canUseDevice = list.stream().filter(item -> "0".equals(item.getSbyxzt())).collect(Collectors.toList());
            //计算可用设备的可用日期
            Map<String, Integer> _empSizeByDay = instance().empSizeByDay;
            canUseDevice.forEach(item -> {
                String[] whArr = item.getWhsj().split("_");
                if (whArr.length < 2) {
                    return;
                }
                DateRange tmpRange = new DateRange(DateUtil.parse(whArr[0], "yyyy/MM/dd"), DateUtil.parse(whArr[1], "yyyy/MM/dd"), DateField.DAY_OF_YEAR);
                List<DateTime> dateTimes = DateUtil.rangeNotContains(dateRange, tmpRange);
                dateTimes.forEach(date -> {
                    String dateStr = date.toString("yyyy/MM/dd");
                    //todo 根据人员请假情况,计算当前日期设备是否可用
                    if (_empSizeByDay.get(dateStr)<=0){
                        return;
                    }
                    if (realDeviceList.get(dateStr) == null) {
                        JSONArray arr = new JSONArray();
                        arr.add(item);
                        realDeviceList.put(dateStr, arr);
                    } else {
                        realDeviceList.getJSONArray(dateStr).add(item);
                    }
                });
            });
        }
    }

    public static void initEmp(Date from,Date to){

    }

    /**
     * 加载设备信息表
     */
    public static void initDevice() {
        instance().deviceList = getBean(EmSbjbbService.class).list(Wrappers.<EmSbjbb>lambdaQuery().like(EmSbjbb::getSzcs,"FP"));
    }

    public synchronized static ComputePlan instance() {
        if (instance == null) {
            instance = new ComputePlan();
        }
        return instance;
    }

    private static RealCapacity initRealCapacity(EmGcrl gcrl, String name, EmSbcnp sbcnp) {
        RealCapacity capacity = new RealCapacity();
        if ("3".equals(gcrl.getPb())) {
            capacity.setCapacity(sbcnp.getDegs() * 7 + sbcnp.getDegsYb() * 7);
        }
        if ("1".equals(gcrl.getPb())) {
            capacity.setCapacity(sbcnp.getDegs() * 7);
        }
        if ("2".equals(gcrl.getPb())) {
            capacity.setCapacity(sbcnp.getDegsYb() * 7);
        }
        capacity.setDeviceName(name);
        capacity.setDate(gcrl.getGl());
        return capacity;
    }

    private static <T> T getBean(Class<T> clazz) {
        return SpringContextHolder.getBean(clazz);
    }
}
